"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function () { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

// Copyright 2020 The OpenZipkin Authors; licensed to You under the Apache License, Version 2.0.
var _require = require('zipkin'),
    Annotation = _require.Annotation,
    InetAddress = _require.InetAddress; // TODO: function wrapPostgres(postgres, options = {})
// as it is easy to get the service name and remote service name wrong when using positional args


module.exports = function zipkinClient(tracer, Postgres) {
  var serviceName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : tracer.localEndpoint.serviceName;
  var remoteServiceName = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'postgres';

  function annotateSuccess(id) {
    tracer.letId(id, function () {
      tracer.recordAnnotation(new Annotation.ClientRecv());
    });
  }

  function annotateError(id, error) {
    tracer.letId(id, function () {
      tracer.recordBinary('error', error.message || String(error));
      tracer.recordAnnotation(new Annotation.ClientRecv());
    });
  }

  function mkZipkinCallback(callback, id) {
    return function zipkinCallback() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      if (args[0]) {
        annotateError(id, args[0]);
      } else {
        annotateSuccess(id);
      }

      callback.apply(this, args);
    };
  }

  var ZipkinPostgresClient = /*#__PURE__*/function (_Postgres$Client) {
    _inherits(ZipkinPostgresClient, _Postgres$Client);

    var _super = _createSuper(ZipkinPostgresClient);

    function ZipkinPostgresClient() {
      _classCallCheck(this, ZipkinPostgresClient);

      return _super.apply(this, arguments);
    }

    return ZipkinPostgresClient;
  }(Postgres.Client);

  var ZipkinPostgres = Object.assign({}, Postgres, {
    Client: ZipkinPostgresClient,
    Pool: function BoundPool(options) {
      return new Postgres.Pool(Object.assign({
        Client: ZipkinPostgresClient
      }, options));
    }
  });
  var actualFn = ZipkinPostgresClient.prototype.query;

  ZipkinPostgresClient.prototype.query = function (config, values, callback) {
    var _this = this;

    var id = tracer.createChildId();
    tracer.letId(id, function () {
      tracer.recordAnnotation(new Annotation.ClientSend());
      tracer.recordAnnotation(new Annotation.ServiceName(serviceName));
      tracer.recordAnnotation(new Annotation.ServerAddr({
        serviceName: remoteServiceName,
        host: new InetAddress(_this.host),
        port: _this.port.toString()
      }));
      tracer.recordRpc("query ".concat(_this.database));
    });

    if (typeof config.submit === 'function') {
      var query = actualFn.call(this, config, values, callback);
      query.on('end', function () {
        annotateSuccess(id);
      });
      query.on('error', function (error) {
        annotateError(id, error);
      });
      return query;
    }

    if (typeof values === 'function') {
      callback = callback || values; // eslint-disable-line no-param-reassign
    }

    if (typeof callback === 'function') {
      return actualFn.call(this, config, values, mkZipkinCallback(callback, id));
    }

    var promise = actualFn.call(this, config, values, callback);
    return promise.then(function () {
      annotateSuccess(id);
      return promise;
    }, function (error) {
      annotateError(id, error);
      return promise;
    });
  };

  return ZipkinPostgres;
};